#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <direct.h>

#include "aplib.h"

#pragma pack(1)

typedef struct _HEADER
{
	DWORD	type;
	DWORD	count;
} HEADER, *PHEADER;

typedef struct _RECORD
{
	DWORD		offset;						//    
	DWORD		type;						//   
	DWORD		sizePack;					//  
	DWORD		sizeUnpack;					//  
	CHAR		name[200];					//   
	CHAR		type2[4];					//   
	DWORD		t1;							//   ,   , 
	DWORD		t2;							//  -  
} RECORD, *PRECORD;

#pragma pack()

BOOL ForceDirectories(char *FileName)
{
	PCHAR	dir;
	PCHAR	curdir;
	PCHAR	newdir;
	int		i;

	dir = new CHAR[_MAX_DIR];
	curdir = new CHAR[_MAX_PATH];
	newdir = new CHAR[_MAX_PATH];

	_splitpath(FileName,NULL,dir,NULL,NULL);
	if( *dir == '\0' )
	{
		return FALSE;
	}
	
	_getcwd(curdir,_MAX_PATH);

	while( *dir )
	{
		i = 0;
		while( *dir != '\\' )
		{
			newdir[i] = *dir;
			dir++;
			i++;
		}
		newdir[i] = '\0';
		dir++;

		if( strlen(newdir) == 0 ) continue;

		_mkdir(newdir);
		if( _chdir(newdir) != 0 ) return FALSE;
	}				
	
	_chdir(curdir);

	delete [] dir;
	delete [] curdir;
	delete [] newdir;

	return TRUE;
}

void UnpackJFL(char *FileName)
{
	CHAR	tmpstr[_MAX_PATH];
	FILE	*in,*out;
	HEADER	header;
	PRECORD records;
	int		i;

	//
	// read index file
	//

	sprintf(tmpstr,"%s.ind",FileName);

	in = fopen(tmpstr,"rb");
	if( !in )
	{
		printf("Error: can't open file %s\n",tmpstr);
		return;
	}

	fread(&header,sizeof(HEADER),1,in);

	records = new RECORD[header.count];
	if( !records )
	{
		printf("Error: can't allocate memory!\n");
		return;
	}

	fread(records,sizeof(RECORD),header.count,in);

	fclose(in);	

	//
	// unpack JFL file
	//
	
	sprintf(tmpstr,"%s.jfl",FileName);
	in = fopen(tmpstr,"rb");
	if( !in )
	{
		printf("Error: can't open file %s\n",tmpstr);
		return;
	}

	for(i = 0; i < header.count; i++)
	{
		PUCHAR	InputBuffer,OutputBuffer;

		printf("Processing %s\n",records[i].name);

		InputBuffer = new UCHAR[ records[i].sizePack ];
		OutputBuffer = new UCHAR[ records[i].sizeUnpack ];

		if( !InputBuffer || !OutputBuffer )
		{
			printf("Error: can't allocate memory!\n");
			return;
		}

		fseek(in,records[i].offset,SEEK_SET);
		fread(InputBuffer,1,records[i].sizePack,in);

		aP_depack_asm(InputBuffer,OutputBuffer);

		ForceDirectories(records[i].name);
		out = fopen(records[i].name,"wb");
		if( !out )
		{
			printf("Error: can't create file %s\n",records[i].name);
			return;
		}
		fwrite(OutputBuffer,1,records[i].sizeUnpack,out);
		fclose(out);

		
		delete [] InputBuffer;
		delete [] OutputBuffer;	
	}
	
	delete [] records;
	fclose(in);
}

void UpdateJFL(char *FileName)
{
	CHAR	tmpstr[_MAX_PATH],jfl_name[_MAX_PATH];
	FILE	*in,*tmp,*out;
	HEADER	header;
	PRECORD records;
	int		i;

	//
	// read index file
	//

	sprintf(tmpstr,"%s.ind",FileName);
	sprintf(jfl_name,"%s.jfl",FileName);

	in = fopen(tmpstr,"rb");
	if( !in )
	{
		printf("Error: can't open file %s\n",tmpstr);
		return;
	}

	fread(&header,sizeof(HEADER),1,in);

	records = new RECORD[header.count];
	if( !records )
	{
		printf("Error: can't allocate memory!\n");
		return;
	}

	fread(records,sizeof(RECORD),header.count,in);

	fclose(in);	


	//
	// update JFL file
	//
	
	sprintf(tmpstr,"%s.jfl.new",FileName);
	out = fopen(tmpstr,"wb");
	if( !out )
	{
		printf("Error: can't create file %s\n",tmpstr);
		return;
	}

	for(i = 0; i < header.count; i++)
	{
		ULONG	WorkBufferSize,InputBufferSize,OutputBufferSize;
		PUCHAR	WorkBuffer,InputBuffer,OutputBuffer;

		printf("Updating %s (%02d%%)\n",records[i].name,((i+1)*100)/header.count);

		in = fopen(records[i].name,"rb");
		if( in )
		{
			fseek(in,0,SEEK_END);
			InputBufferSize = ftell(in);
			fseek(in,0,SEEK_SET);

			InputBuffer = new UCHAR[InputBufferSize];
			WorkBufferSize = aP_workmem_size(InputBufferSize);
			WorkBuffer = new UCHAR[WorkBufferSize];
			OutputBufferSize = aP_max_packed_size(InputBufferSize);
			OutputBuffer = new UCHAR[OutputBufferSize];

			if( !InputBuffer || !OutputBuffer || !WorkBuffer )
			{
				printf("Error: can't allocate memory!\n");
				return;
			}
		
			fread(InputBuffer,1,InputBufferSize,in);
			fclose(in);

	        OutputBufferSize = aP_pack(
				InputBuffer,
				OutputBuffer,
				InputBufferSize,
				WorkBuffer,
				NULL);

			if( !OutputBufferSize )
			{
				printf("Error while packing!\n");
				return;
			}

			delete [] WorkBuffer;
			delete [] InputBuffer;
		}
		else
		{
			//
			// update from original jfl file
			//
			printf("read from old file!\n");
		    in = fopen(jfl_name,"rb");
			fseek(in,records[i].offset,SEEK_SET);
			InputBufferSize = records[i].sizeUnpack;
			OutputBufferSize = records[i].sizePack;
			OutputBuffer = new UCHAR[records[i].sizePack];
			fread(OutputBuffer,1,OutputBufferSize,in);
			fclose(in);
		}

		records[i].offset = ftell(out);
		records[i].sizePack = OutputBufferSize;
		records[i].sizeUnpack = InputBufferSize;

/*
		for(int j = 0; j < 8; j+= 2 )
		{
			PCHAR str1,str2;

			if( (str1 = strstr(records[i].name,dir_to_replace[j])) != NULL )
			{
				str2 = strstr(str1+1,"\\") + 1;
				memset(tmpstr,0,_MAX_PATH);
				strncpy(tmpstr,records[i].name,str1 - records[i].name);

//				printf("was: %s\n",records[i].name);
//				printf("str1: %s\n",str1);
//				printf("str2: %s\n",str2);
//				printf("tmpstr: %s\n",tmpstr);
				
				strcat(tmpstr,dir_to_replace[j+1]);
				strcat(tmpstr,str2);
				memset(records[i].name,0,sizeof(records[i].name));
				strcpy(records[i].name,tmpstr);
//				printf("will: %s\n",records[i].name);
			}

		}
*/


		fwrite(OutputBuffer,1,OutputBufferSize,out);

		delete [] OutputBuffer;
	}

	fclose(out);

	//
	// update index file
	//

	sprintf(tmpstr,"%s.ind.new",FileName);
	out = fopen(tmpstr,"wb");
	fwrite(&header,sizeof(HEADER),1,out);
	fwrite(records,sizeof(RECORD),header.count,out);
	fclose(out);

	delete [] records;
}

void main(int argc,char *argv[])
{
	if( argc < 3 )
	{
		printf("Industry Giant II resource unpacker\n");
		return;
	}
	if( stricmp(argv[1],"x") == 0 )
	{
		UnpackJFL(argv[2]);
	}
	if( stricmp(argv[1],"u") == 0 )
	{
		UpdateJFL(argv[2]);
	}
}